// ==UserScript==
// @name          Editable Web
// @namespace     http://www.wischik.com/lu/programmer/editable-web/
// @description   Makes any page editable within the web-browser
// @include       *.html
// @include       */
// @exclude       *.php*
// @exclude       *.asp*
// ==/UserScript==

window.addEventListener('load',EditableOnLoad,true);


// EditableOnLoad: this routine is called by onLoad.
// * If the web page already had a script tag for "editable.js", then we do nothing
// * Otherwise we put a small "editing mark" at the top right of the page, id="editable_loader"
// * If this browser session had the editable cookie, then we load the editor immediately.
// Notes: once the editable web proper (editable1.js) has loaded, it will set the cookie
// and delete the editing mark.
//

function EditableOnLoad()
{ var ss=document.getElementsByTagName("script");
  for (var i=0; i<ss.length; i++) if (ss[i].src.match(/editable1?.js/)) return;
  var e=document.createElement('div');
  e.innerHTML='<a id="editable_loader" style="top:0;right:3px;position:absolute;font-family:Courier New;font-size:10pt;text-decoration:none;opacity:0.4;" href="javascript:editable1(true);">&dagger;</span>';
  document.body.appendChild(e);
  document.getElementById("editable_loader").addEventListener("mousedown",editable1,true);
  if (document.cookie.match(/^(.*;( )?)?(editable=on)(;.*)?$/)) editable1();
}


// editable1: this routine's mission is to seek out and load editable1.js.
// * It uses the "editing mark", id="editable_loader", as a little animated progress indicator.
// * It fires off repeated ajax requests to look for editable/editable.php in various places.
//   e.g. if we're at "http://a.com/b/c/d.html" it will look in "/b/c/editable/editable.php"
//   then in "/b/editable/editable.php" then in "/editable/editable.php" and then it'll give up.
//   A return code of 404 means that we try the next place.
// * The request is just a dummy one, "editable.php?get&path=...&file=..." to try and get
//   the current page that we're currently viewing. Editable.php might give back various
//   other (non-404) response codes, along with error messages, that warn of misconfiguration.
//   e.g. the file is not editable by the webserver. We suppress error messages if we
//   were launched by an auto-load.
// * If everything's okay, we insert a <script src=".../editable/editable1.js"> tag to load
//   up the main Editable Web. This has its own functionality: it deletes any editing mark
//   (id="editable_loader") and it saves a cookie.
//

function editable1(manual)
{ var xmlhttp = new XMLHttpRequest();
  var r = urlsplit(location.href), filepath=r.path, filename=r.file;
  var editpath=filepath;

  var c=document.getElementById("editable_loader"); c.innerHTML='|';
  function animate()
  { c.innerHTML={'\\':'|', '-':'\\', '/':'-', '|':'/', '\u2020':'\u2020'}[c.innerHTML];
    if (c.parentNode) setTimeout(animate,300);
  }
  animate();

  function cc()
  { var req = editpath+"editable/editable.php?get&innerouter=outer&path="+encodeURIComponent(filepath)+"&file="+encodeURIComponent(filename)+"&xpath=/&random="+Math.floor(Math.random()*99999);
    xmlhttp.open('get',req,true);
    xmlhttp.onreadystatechange = function()
    { if (xmlhttp.readyState!=4) return;
      if (xmlhttp.status==404)
      { if (editpath!='/') {editpath=editpath.replace(/^(.*\/)[^\/]*\/$/,'$1'); cc(); return;}
        c.innerHTML='&dagger;'; if (!manual) return;
        var r=confirm("Editable Web not installed on this website.\nDo you want to download it?");
        if (r) window.location.href="http://www.wischik.com/lu/programmer/editable-web/download.html";
        return;
      }
      if (xmlhttp.status!=200)
      { c.innerHTML='&dagger'; if (!manual) return;
        alert("Editable Web error:\n"+xmlhttp.responseText.replace(/<[^>]+>/ig,' '));
        return;
      }
      var e=document.createElement('script'); e.src=editpath+"editable/editable1.js";
      document.getElementsByTagName("head")[0].appendChild(e);
    }
    try {xmlhttp.send(null);}
    catch (e) {c.innerHTML='&dagger'; if (manual) alert("Editable Web not found on this site");}
  }
  cc();
}


function urlsplit(url)
{ var m = /((http|file):\/\/[^\/]*)?([^\?#]*\/)?([^\?\/#]*)(#[^\?]*)?(\?.*)?/i.exec(url);
  if (!m) return null; else return {'url':url, 'site':m[1]?m[1]:'', 'path':m[3]?m[3]:'', 'file':m[4]?m[4]:'', 'section':m[5]?m[5]:'', 'query':m[6]?m[6]:''};
}
